import numpy as np

from osol.algorithms.abstract import OptimizationAlgorithm
from osol.tools.vectors import bound_vector, generate_vector_in_area


class BigBangBigCrunch(OptimizationAlgorithm):
    def __init__(self, N, radius):
        self.N = N
        self.radius = radius

    def _calculate_values(self, f):
        self.values = [f(x) for x in self.points]

    def _get_center_of_mass(self):
        return self.points[np.where(self.values == np.min(self.values))[0][0]]

    def _generate_delta(self, x):
        return self.radius * np.random.normal(size=x.size) / self.iteration_number

    def initialize(self, f, search_area):
        self.iteration_number = 1
        self.points = [generate_vector_in_area(search_area) for _ in range(self.N)]
        self._calculate_values(f)

    def iterate(self, f, search_area):
        center_of_mass = self._get_center_of_mass()

        self.points = [center_of_mass + self._generate_delta(x) for x in self.points]
        self.points = [bound_vector(x, search_area) for x in self.points]
        self.points.append(center_of_mass)
        self._calculate_values(f)

        self.iteration_number += 1

    def terminate(self, _, __):
        return self._get_center_of_mass()



import numpy as np
from scipy.optimize import approx_fprime

from osol.algorithms.abstract import OptimizationAlgorithm
from osol.tools.vectors import bound_vector, generate_vector_in_area


class NumericalGradientDescent(OptimizationAlgorithm):
    def __init__(self, radius, eps=1e-7):
        self.radius = radius
        self.eps = eps

    def initialize(self, f, search_area):
        self.x = generate_vector_in_area(search_area)
        self.f_x = f(self.x)

    def iterate(self, f, search_area):
        gradient = approx_fprime(self.x, f, epsilon=self.eps)
        gradient_length = np.linalg.norm(gradient)
        if gradient_length > 0:
            gradient /= gradient_length

        x_new = self.x - np.random.uniform(0.0, self.radius) * gradient
        x_new = bound_vector(x_new, search_area)
        f_x_new = f(x_new)
        if f_x_new < self.f_x:
            self.x = x_new
            self.f_x = f_x_new

    def terminate(self, _, __):
        return self.x



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['osol', 'osol.algorithms', 'osol.tools']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.20.2,<2.0.0', 'scipy>=1.6.2,<2.0.0']

setup_kwargs = {
    'name': 'osol',
    'version': '0.0.9',
    'description': '',
    'long_description': None,
    'author': 'wol4aravio',
    'author_email': 'panovskiy.v@yandex.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.0,<3.9.0',
}


setup(**setup_kwargs)



# pylint: disable=protected-access

import numpy as np


class FeldbaumTestFunctions:
    @staticmethod
    def _generate_function(a, b, c, p):
        return lambda x: np.sum(a * np.abs(x - c) ** p) + b

    def _compose_function_parts(self):
        self._functions = [
            FeldbaumTestFunctions._generate_function(
                self._a[i, :], self._b[i, :], self._c[i, :], self._p[i, :]
            )
            for i in range(self._N)
        ]

    def __init__(self, N, dim):
        self._N = N
        self._dim = dim
        self._a = np.random.uniform(low=0.0, high=5.0, size=(N, dim))
        self._b = np.random.uniform(low=-10.0, high=10.0, size=(N, 1))
        self._c = np.random.uniform(low=-10.0, high=10.0, size=(N, dim))
        self._p = np.random.uniform(low=0.0, high=5.0, size=(N, dim))
        self._functions = None
        self._compose_function_parts()

    def __call__(self, x):
        return np.min([f(x) for f in self._functions])


def smoke_check(algorithm, number_of_iterations, eps=1e-2):
    left, right = -10, 10
    search_area = np.array([[left, right], [left, right]])
    x_opt = np.random.uniform(left, right, 2)
    x_found = algorithm.optimize(
        lambda x: np.sum((x - x_opt) ** 2), search_area, number_of_iterations
    )
    return np.linalg.norm(x_found - x_opt) < eps


def feldbaum_check(algorithm, number_of_iterations, eps=1e-2):
    left, right = -10, 10
    search_area = np.array([[left, right], [left, right]])
    f = FeldbaumTestFunctions(N=3, dim=2)
    y_opt = np.min(f._b)
    y_opt_loc = np.where(f._b == y_opt)[0][0]
    x_opt = f._c[y_opt_loc, :]
    x_found = algorithm.optimize(f, search_area, number_of_iterations)
    return np.linalg.norm(x_found - x_opt) < eps
